#include <iostream>
#include <vector>
#include <string>
using namespace std;

// ======================================================================
struct Mot {
  string mot;
  bool feminin; // ou (false) masculin
  string pluriel; // "" pour les invariables, suffixe sur 1 char ou mot entier sinon
};

// ======================================================================
struct Ingredient {
  Mot nom;
  double qtt;
  Mot unite;
};

// ======================================================================
struct Recette {
  Mot titre;
  unsigned int nb_convives;     // nombre de personnes pour la recette de base
  vector<Ingredient> ingredients;
};

// ======================================================================
unsigned int demander_nb_convives()
{
  int nb_convives(0); // je garde ici 'int' tant qu'on ne sait pas gérer les entrées/sorties
  do {
    cout << "Entrez le nombre de personnes conviées à la fondue : ";
    cin >> nb_convives;
  } while (nb_convives < 1);
  return nb_convives;
}

// ======================================================================
// « adapter » ou « mettre à jour »
// Deux choix possibles :
//     void adapter(Fondue& f);
// ou  Fondue adapter(Fondue const& f);
Recette adapter(Recette recette, unsigned int nb_convives)
{
  double ratio(nb_convives);      // Notez le 'double' ici...
  ratio /= recette.nb_convives ; // ... pour éviter une division entière.

  // Mise à jour des quantités en fonction du nombre de convives :
  recette.nb_convives = nb_convives;
  
  // on modifie ==> donc ne pas oublier le & !
  for (auto& ingredient : recette.ingredients) {
    ingredient.qtt *= ratio;
  }

  return recette;
}

// ======================================================================
bool est_voyelle(char c) {
  return "aeiouyAEIOUY"s.find(c) != string::npos;
  // pour les caractères accentués, c'est franchement plus compliqué (encodage)
}

// ======================================================================
void afficher(Mot const& m, bool pluriel = false)
{
  if (pluriel and not m.pluriel.empty()) {
    if (m.pluriel.size() == 1) {
      cout << m.mot << m.pluriel;
    } else {
      cout << m.pluriel;
    }
  } else {
    // singulier
    cout << m.mot;
  }
}

// ======================================================================
void afficher(Ingredient const& i)
{
  cout << i.qtt << ' ';
  afficher(i.unite, i.qtt > 1.0);
  if (est_voyelle(i.nom.mot.front())) {
    cout << " d'";
  } else {
    cout << " de ";
  }
  afficher(i.nom);
}

// ======================================================================
void afficher(Recette const& r)
{
  cout << "Pour faire un";
  if (r.titre.feminin) cout << 'e';
  cout << " ";
  afficher(r.titre);
  cout << " pour " << r.nb_convives << " personne";
  if (r.nb_convives > 1) cout << 's';
  cout <<  ", " << endl;
  cout << "il vous faut : " << endl;
  for (auto& ingredient : r.ingredients) {
    cout << " - ";
    afficher(ingredient);
    cout << endl;
  }
}

// ======================================================================
int main()
{
  const Recette r = {
    { "fondue fribourgeoise", true, "fondues fribourgeoises" },
    4, // nb de personnes
    { { { "fromage", false, "s"    }, 800.0, { "g",      false, ""  } },
      { { "eau",     true,  "x"    },   2.0, { "dl",     false, ""  } },
      { { "ail",     false, "aulx" },   2.0, { "gousse", true,  "s" } },
      { { "pain",    false, "s"    }, 400.0, { "g",      false, ""  } }
    }
  };

  afficher( adapter(r, demander_nb_convives()));

  return 0;
}

